@@ -2,9 +2,10 @@ import gtk
 import gobject
 import pango
 import gconf
+import logging
 
 import deskbar
-from deskbar.ui.cuemiac.CuemiacItems import CuemiacCategory, Nest
+from deskbar.ui.cuemiac.CuemiacItems import CuemiacCategory
 
 class CellRendererCuemiacCategory (gtk.CellRendererText):
 	"""
@@ -32,13 +33,24 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
 		self.__category_header = None
 		self.__match_count = 0
 		
-		# Obtain theme font and set it to bold and decrease size 2 points
+		# Grab some default theme settings
+		# they are probably incorrect, but we reset
+		# them on each render anyway.
 		style = gtk.Style ()
 		self.header_font_desc = style.font_desc
 		self.header_font_desc.set_weight (pango.WEIGHT_BOLD)
 		self.header_font_desc.set_size (self.header_font_desc.get_size () - pango.SCALE *2)
 		self.header_bg = style.base [gtk.STATE_NORMAL]
 	
+	def set_style (self, widget):
+		"""
+		Apply the style from widget, to this cellrenderer
+		"""
+		self.header_font_desc = widget.style.font_desc
+		self.header_font_desc.set_weight (pango.WEIGHT_BOLD)
+		self.header_font_desc.set_size (self.header_font_desc.get_size () - pango.SCALE *2)
+		self.header_bg = widget.style.base [gtk.STATE_NORMAL]
+	
 	def do_render (self, window, widget, background_area, cell_area, expose_area, flags):
 		if not self.get_property ("category-header"):
 			gtk.CellRendererText.do_render (self, window, widget, background_area, cell_area, expose_area, flags)
@@ -50,6 +62,9 @@ class CellRendererCuemiacCategory (gtk.CellRendererText):
 		Renders the category title from the "category-header" property and displays a rigth aligned
 		hit count (read from the "match-count" property).
 		"""
+		# Ensure we have fresh style settings
+		self.set_style (widget)
+		
 		ctx = window.cairo_create ()
 		
 		# Set up a pango.Layout for the category title
@@ -120,9 +135,9 @@ class CuemiacTreeView (gtk.TreeView):
 		
 		self.connect ("cursor-changed", self.__on_cursor_changed)
 		self.set_property ("headers-visible", False)
-		self.connect ("row-activated", lambda w,p,c: self.__on_activated())
-		self.connect ("button-press-event", self.__on_button_press_event)
-		
+
+		self.connect ("row-activated", self.__on_activated)
+		self.connect ("button-press-event", self.__on_button_press)		
 		self.connect ("key-press-event", self.__on_key_press)
 				
 		self.set_enable_search (False)
@@ -131,23 +146,6 @@ class CuemiacTreeView (gtk.TreeView):
 		# gtkTreeView does not use normal gtk DnD api.
 		# it uses the api from hell
 		
-		gtk_style = gtk.Style ()
-		self.header_bg = gtk_style.bg[gtk.STATE_NORMAL]
-		self.match_bg = gtk_style.base [gtk.STATE_NORMAL]
-		
-		# Stuff to handle persistant expansion states.
-		# A category will be expanded if it's in __collapsed_rows
-		# a Nest will never be expanded (this is not a bug, but policy)
-		self.__collapsed_rows = deskbar.GCONF_CLIENT.get_list(deskbar.GCONF_COLLAPSED_CAT, gconf.VALUE_STRING)
-		deskbar.GCONF_CLIENT.notify_add(deskbar.GCONF_COLLAPSED_CAT, lambda x, y, z, a: self.__on_config_expanded_cat(z.value))
-		
-		self.connect ("row-expanded", self.__on_row_expanded, model)
-		self.connect ("row-collapsed", self.__on_row_collapsed, model)
-		model.connect ("category-added", self.__on_category_added)
-		model.connect ("nest-added", self.__on_nest_added)
-		
-		self.qstring = ""
-
 	def is_ready (self):
 		""" Returns True if the view is ready for user interaction """
 		num_children = self.get_model().iter_n_children (None)
@@ -156,9 +154,6 @@ class CuemiacTreeView (gtk.TreeView):
 	def clear (self):
 		self.model.clear ()
 	
-	def set_query_string (self, qstring):
-		self.qstring = qstring
-	
 	def last_visible_path (self):
 		"""Returns the path to the last visible cell."""
 		model = self.get_model ()
@@ -180,57 +175,99 @@ class CuemiacTreeView (gtk.TreeView):
 		# Go to last child of category
 		num_children = model.iter_n_children (iter)
 		iter = model.iter_nth_child (iter, num_children - 1)
-		
-		# The child might be a Nest. If it's expanded
-		# go into that. If not return.
-		if not self.row_expanded (model.get_path (iter)):
-			return model.get_path (iter)
-			
-		num_children = model.iter_n_children (iter)
-		iter = model.iter_nth_child (iter, num_children - 1)
 		return model.get_path (iter)
+
+	def move_cursor_up_down (self, count):
+		"""
+		Move cursor one step up or down in the tree.
+		@param count: 1 for up/next, -1 for down/previous.
+		@return: True if the move was succesful.
+		"""
+		# The implementation of this method is one big hack. 
+		# This method is needed because gtk.TreeView does not handle events
+		# correctly when unfocused (and we need to forward events to it).
+		# See bug #326254.
+		# We cannot simply port the navigational model because 
+		# gtk.TreeView uses a GtkRBTree, which is not a part of the public api,
+		# to implement the navigational model.
+		path, col = self.get_cursor ()
+		rect = self.get_cell_area (path, col)
+		
+		new_cell_y = 0
+		if count == -1:
+			# Select a point in the cell above the cursor
+			new_cell_y = rect.y - 5
+		else:
+			if count != 1:
+				logging.warning("CuemiacTreeView - in move_cursor_up_down, the count must be 1 or -1.")
+			# Select a point in the cell below the cursor
+			new_cell_y = rect.height + rect.y + 5
+				
+		# Select the cell meeting the point (rect.x, new_cell_y)
+		cell_ctx = self.get_path_at_pos (rect.x, new_cell_y)
+		if cell_ctx is None:
+			return False
+		path, col, x, y = cell_ctx
+		self.set_cursor (path, col)
+		
+		return True
+		
+	def focus_bottom_match (self):
+		last = self.last_visible_path ()
+		self.set_cursor (last)
 	
-	def __on_config_expanded_cat (self, value):
-		if value != None and value.type == gconf.VALUE_LIST:
-			self.__collapsed_rows = [h.get_string() for h in value.get_list()]
-			
-	def __on_row_expanded (self, widget, iter, path, model):
-		idx = model[iter][model.MATCHES].get_id ()
-		if idx in self.__collapsed_rows:
-			self.__collapsed_rows.remove (idx)
-			deskbar.GCONF_CLIENT.set_list(deskbar.GCONF_COLLAPSED_CAT, gconf.VALUE_STRING, self.__collapsed_rows)
-		
-	def __on_row_collapsed (self, widget, iter, path, model):
-		idx = model[iter][model.MATCHES].get_id ()
-		self.__collapsed_rows.append (idx)
-		deskbar.GCONF_CLIENT.set_list(deskbar.GCONF_COLLAPSED_CAT, gconf.VALUE_STRING, self.__collapsed_rows)
+	def focus_top_match (self):
+		model = self.get_model ()
+		first = model.get_path(model.get_iter_first())
+		self.set_cursor (first)
+
+	def coord_is_category (self, x, y):
+		path_ctx = self.get_path_at_pos(int(x), int(y))
+		if path_ctx is None:
+			return False
+		path, col, x, y = path_ctx
+		model = self.get_model()
+		match = model[model.get_iter(path)][model.MATCHES]
+		if match.__class__ == CuemiacCategory:
+			return True
+		else:
+			return False
+
+	def __on_button_press (self, treeview, event):
+		# We want to activate items on single click
+		path_ctx = self.get_path_at_pos (int(event.x), int(event.y))
+		if path_ctx is not None:
+			path, col, x, y = path_ctx
+			model = self.get_model ()
+			match = model[model.get_iter(path)][model.MATCHES]
+			if match.__class__ != CuemiacCategory:
+				self.emit ("row-activated", path, col)
+				#self.emit ("match-selected", match)
 	
-	def __on_category_added (self, widget, cat, path):
-		if cat.get_id() not in self.__collapsed_rows:
-			self.expand_row (path, False)
-		
-	def __on_nest_added (self, widget, cat, path):
-		pass
-		
+#	def __on_config_expanded_cat (self, value):
+#		if value != None and value.type == gconf.VALUE_LIST:
+#			self.__collapsed_rows = [h.get_string() for h in value.get_list()]
+				
 	def __on_cursor_changed (self, view):
 		model, iter = self.get_selection().get_selected()
 	
 	def __get_match_icon_for_cell (self, column, cell, model, iter, data=None):
 	
 		match = model[iter][model.MATCHES]
+
 		
 		if match.__class__ == CuemiacCategory:
 			cell.set_property ("pixbuf", None)
-			cell.set_property ("cell-background-gdk", self.header_bg)
+			cell.set_property ("cell-background-gdk", self.style.bg[gtk.STATE_NORMAL])
 			
+		elif match.__class__== tuple:
+			cell.set_property ("cell-background-gdk", self.style.base[gtk.STATE_NORMAL])
+			qstring, match_obj = match
+			cell.set_property ("pixbuf", match_obj.get_icon())
 		else:
-			cell.set_property ("cell-background-gdk", self.match_bg)
-			if match.__class__ == Nest:
-				cell.set_property ("pixbuf", None)		
-			else:
-				qstring, match_obj = match
-				cell.set_property ("pixbuf", match_obj.get_icon())
-
+			logging.error("See bug 359251 and report this output: Match object of unexpected type: %r - %r" % (match.__class__, match))
+			cell.set_property ("cell-background-gdk", self.style.base[gtk.STATE_NORMAL])
+			cell.set_property ("pixbuf", None)
 		
 	def __get_match_title_for_cell (self, column, cell, model, iter, data=None):
 	
@@ -238,7 +275,7 @@ class CuemiacTreeView (gtk.TreeView):
 		
 		if match.__class__ == CuemiacCategory:
 			# Look up i18n category name
-			cell.set_property ("cell-background-gdk", self.header_bg)
+			cell.set_property ("cell-background-gdk", self.style.bg[gtk.STATE_NORMAL])
 			cell.set_property ("height", 20)
 			cell.set_property ("category-header", match.get_name())
 			cell.set_property ("match-count", match.get_count ())
@@ -246,20 +283,21 @@ class CuemiacTreeView (gtk.TreeView):
 		
 		cell.set_property ("category-header", None)
 		cell.set_property ("height", -1)
-		cell.set_property ("cell-background-gdk", self.match_bg)
-		
-		if match.__class__ == Nest:
-			cell.set_property ("markup", match.get_verb() % match.get_name())
-			return
-		
+		cell.set_property ("cell-background-gdk", self.style.base[gtk.STATE_NORMAL])
+				
 		cell.set_property ("markup", model[iter][model.ACTIONS])
 
-	def __on_activated (self, path=None):
-		if path == None:
-			model, iter = self.get_selection().get_selected()
-		else:
-			model, iter = self.get_model(), path
+	def __on_activated (self, treeview, path, col):
+		model = self.get_model()
+		iter = model.get_iter (path)
 		match = model[iter][model.MATCHES]
+		
+		# Check if this s really a match and not just
+		# a category
+		if match.__class__ == CuemiacCategory:
+			return
+			
+		# So we have a Match, tell the world
 		self.emit ("match-selected", match)
 		
 		
@@ -269,9 +307,9 @@ class CuemiacTreeView (gtk.TreeView):
 		if iter is None:
 			return False
 		match = model[iter][model.MATCHES]
-		# If this is a category or nest, toggle expansion state
+		# If this is a category, toggle expansion state
 		if event.keyval in self.activation_keys:
-			if match.__class__ == Nest or match.__class__ == CuemiacCategory:
+			if match.__class__ == CuemiacCategory:
 				path = model.get_path (iter)
 				if self.row_expanded (path):
 					self.collapse_row (path)
@@ -280,11 +318,6 @@ class CuemiacTreeView (gtk.TreeView):
 				return True
 			
 		return False
-	
-	def __on_button_press_event(self, widget, event):
-		path = self.get_path_at_pos(int(event.x), int(event.y))
-		if path != None:
-			self.__on_activated(path[0])
 
 if gtk.pygtk_version < (2,8,0):	
 	gobject.type_register (CuemiacTreeView)
